@@ -258,7 +258,6 @@ S_g_object_set_property(USER_OBJECT_ s_object, USER_OBJECT_ s_property_name, USE
         GValue value = { 0, };
 
         USER_OBJECT_ _result = NULL_USER_OBJECT;
-
 		R_setGValueForProperty(&value, G_OBJECT_GET_CLASS(object), property_name, s_value);
 		
         g_object_set_property(object, property_name, &value);
@@ -370,8 +369,8 @@ parseConstructorParams(GType obj_type, char **prop_names, GParameter *params,
         g_value_init(&params[param_i].value, spec->value_type);
         if (R_setGValueFromSValue(&params[param_i].value, args[arg_i]) == -1) {
             int i;
-            fprintf(stderr, "Could not convert property '%s' of type '%s'\n",
-                         prop_names[arg_i], g_type_name(spec->value_type));
+            warning("Could not convert property '%s' of type '%s'",
+                    prop_names[arg_i], g_type_name(spec->value_type));
             g_type_class_unref(oclass);
             for (i = 0; i < param_i; ++i)
                 g_value_unset(&params[i].value);
@@ -1173,14 +1172,15 @@ R_GClosureMarshal(GClosure *closure, GValue *return_value, guint n_param_values,
         }
     } else {
         e = cbdata->function;
-        if(cbdata->data && cbdata->data != NULL_USER_OBJECT && TYPEOF(cbdata->data) == ENVSXP) {
-            envir = cbdata->data;
-        }
+        if(cbdata->data && cbdata->data != NULL_USER_OBJECT &&
+           TYPEOF(cbdata->data) == ENVSXP)
+          envir = cbdata->data;
     }
 
     val = R_tryEval(e, envir, &errorOccurred);
 
-    if(errorOccurred || !return_value || G_VALUE_TYPE(return_value) == G_TYPE_NONE ||
+    if(errorOccurred || !return_value ||
+       G_VALUE_TYPE(return_value) == G_TYPE_NONE ||
        G_VALUE_TYPE(return_value) == G_TYPE_INVALID) 
     {
         UNPROTECT(numProtects);
@@ -1271,7 +1271,6 @@ int
 R_setGValueFromSValue(GValue *value, USER_OBJECT_ sval) {
     GValue *raw = createGValueFromSValue(sval);
     int ret = 0;
-
 	if (raw && g_value_type_compatible(G_VALUE_TYPE(raw), G_VALUE_TYPE(value)))
         g_value_copy(raw, value);
     else if (raw && g_value_type_transformable(G_VALUE_TYPE(raw), G_VALUE_TYPE(value)))
@@ -1325,17 +1324,26 @@ R_setGValueFromSValue(GValue *value, USER_OBJECT_ sval) {
 			else g_value_set_boxed(value, getPtrValue(sval));
 		break;
 		case G_TYPE_POINTER:
-			g_value_set_pointer(value, sval == NULL_USER_OBJECT ? NULL : getPtrValue(sval));
-		break;
+                  /*
+                    g_value_set_pointer(value, sval == NULL_USER_OBJECT
+                    ? NULL : getPtrValue(sval));
+                    break;
+                  */
 		case G_TYPE_INTERFACE:
 		case G_TYPE_OBJECT:
-			g_value_set_object(value, sval == NULL_USER_OBJECT ? NULL : getPtrValue(sval));
-		break;
+                  /*
+                    g_value_set_object(value, sval == NULL_USER_OBJECT
+                    ? NULL : getPtrValue(sval));
+                  */
+                  /* If we get here, we know that initGValueFromSValue()
+                     found that 'sval' is not an externalptr */
+                  PROBLEM "Cannot set pointer value from non-externalptr\n"
+                    ERROR;
 		case G_TYPE_INVALID:
-			fprintf(stderr, "Attempt to set invalid type\n");fflush(stderr);
+                  PROBLEM "Attempt to set invalid type\n" ERROR;
 		break;
 		case G_TYPE_NONE:
-			fprintf(stderr, "None type\n");fflush(stderr);
+                  PROBLEM "Attempt to set none type\n" ERROR;
 		break;
 		default:
 			PROBLEM "got an unknown/unhandled type named: %s\n", g_type_name(G_VALUE_TYPE(value))
@@ -1380,7 +1388,7 @@ R_setGValueFromSValue(GValue *value, USER_OBJECT_ sval)
         g_value_set_string(raw, asCString(sval));
     break;
      default:
-     fprintf(stderr, "Unhandled R type %d\n", TYPEOF(sval));fflush(stderr);
+     warning("Unhandled R type %d", TYPEOF(sval));
     }
 
     if (g_value_type_compatible(G_VALUE_TYPE(raw), G_VALUE_TYPE(value))) {
@@ -1389,7 +1397,8 @@ R_setGValueFromSValue(GValue *value, USER_OBJECT_ sval)
     else if (g_value_type_transformable(G_VALUE_TYPE(raw), G_VALUE_TYPE(value)))
         g_value_transform(raw, value);
     else {
-        fprintf(stderr, "Could not set GValue type %s\n", g_type_name(G_VALUE_TYPE(value)));fflush(stderr);
+        warning("Could not set GValue type %s",
+                g_type_name(G_VALUE_TYPE(value)));
         ret = -1;
     }
 
@@ -1406,6 +1415,8 @@ asRGValue(const GValue *value)
 {
     USER_OBJECT_ ans = NULL_USER_OBJECT;
 
+    g_return_val_if_fail(G_IS_VALUE(value), ans);
+    
     switch(G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(value))) {
       case G_TYPE_CHAR:
       {
@@ -1463,6 +1474,8 @@ asRGValue(const GValue *value)
           ans = toRGdkEvent(g_value_get_boxed(value), FALSE);
         else if (G_VALUE_TYPE(value) == R_GTK_TYPE_SEXP)
           ans = g_value_get_boxed(value);
+        else if (G_VALUE_TYPE(value) == GDK_TYPE_COLOR)
+          ans = asRGdkColor(g_value_get_boxed(value));
         else ans = toRPointer(g_value_get_boxed(value), G_VALUE_TYPE_NAME(value));
       break;
 
@@ -1474,11 +1487,11 @@ asRGValue(const GValue *value)
       break;
 
       case G_TYPE_INVALID:
-      fprintf(stderr, "Attempt to get invalid type\n");fflush(stderr);
+        warning("Attempt to get invalid type");
       break;
 
       case G_TYPE_NONE:
-      fprintf(stderr, "None type\n");fflush(stderr);
+        warning("None type");
       break;
 
       case G_TYPE_OBJECT:
@@ -1514,35 +1527,44 @@ createGValueFromSValue(USER_OBJECT_ sval) {
 GType
 getSValueGType(USER_OBJECT_ sval)
 {
-	switch(TYPEOF(sval)) {
-     case LGLSXP:
-        return(G_TYPE_BOOLEAN);
-	 break;
-	 case INTSXP:
-	{
-		USER_OBJECT_ levels;
-		if ((levels = getAttrib(sval, install("levels"))) != NULL_USER_OBJECT)
-			return(G_TYPE_STRING);
-		else return(G_TYPE_INT);
-	}
+  switch(TYPEOF(sval)) {
+  case LGLSXP:
+    return(G_TYPE_BOOLEAN);
     break;
-      case REALSXP:
-		return(G_TYPE_DOUBLE);
-	break;
-      case EXTPTRSXP:
-	  {
-		GType type = g_type_from_name(asCString(GET_CLASS(sval)));
-		if (type == G_TYPE_INVALID)
-			return(G_TYPE_POINTER);
-        else return(type);
-	  }
-    break;
-      case STRSXP:
-	  case CHARSXP:
+  case INTSXP:
+    {
+      USER_OBJECT_ levels;
+      if ((levels = getAttrib(sval, install("levels"))) != NULL_USER_OBJECT)
         return(G_TYPE_STRING);
-    break;
+      else return(G_TYPE_INT);
     }
-	return(G_TYPE_INVALID);
+    break;
+  case REALSXP:
+    return(G_TYPE_DOUBLE);
+    break;
+  case EXTPTRSXP:
+    {
+      GType type = g_type_from_name(asCString(GET_CLASS(sval)));
+      if (type == G_TYPE_INVALID)
+        return(G_TYPE_POINTER);
+      else return(type);
+    }
+    break;
+  case STRSXP:
+  case CHARSXP:
+    return(G_TYPE_STRING);
+    break;
+  case VECSXP:
+    if (GET_LENGTH(sval)) {
+      GType first = getSValueGType(VECTOR_ELT(sval, 0));
+      for (int i = 1; i < GET_LENGTH(sval); i++)
+        if (getSValueGType(VECTOR_ELT(sval, i)) != first)
+          return(G_TYPE_INVALID);
+      return(first);
+    }
+    break;
+  }
+  return(G_TYPE_INVALID);
 }
 
 gboolean
@@ -1583,12 +1605,12 @@ initGValueFromSValue(USER_OBJECT_ sval, GValue *raw) {
 
 gboolean
 initGValueFromVector(USER_OBJECT_ sval, gint n, GValue *raw) {
-	switch(TYPEOF(sval)) {
-    case LGLSXP:
-      g_value_init(raw, G_TYPE_BOOLEAN);
-      g_value_set_boolean(raw, LOGICAL_DATA(sval)[n]);
+  switch(TYPEOF(sval)) {
+  case LGLSXP:
+    g_value_init(raw, G_TYPE_BOOLEAN);
+    g_value_set_boolean(raw, LOGICAL_DATA(sval)[n]);
     break;
-    case INTSXP:
+  case INTSXP:
     {
       USER_OBJECT_ levels;
       if ((levels = getAttrib(sval, install("levels"))) != NULL_USER_OBJECT) {
@@ -1605,20 +1627,23 @@ initGValueFromVector(USER_OBJECT_ sval, gint n, GValue *raw) {
       }
     }
     break;
-    case REALSXP:
-      g_value_init(raw, G_TYPE_DOUBLE);
-      g_value_set_double(raw, NUMERIC_DATA(sval)[n]);
+  case REALSXP:
+    g_value_init(raw, G_TYPE_DOUBLE);
+    g_value_set_double(raw, NUMERIC_DATA(sval)[n]);
     break;
-	  case STRSXP:
-	  case CHARSXP:
-      g_value_init(raw, G_TYPE_STRING);
-      g_value_set_string(raw, asCString(STRING_ELT(sval, n)));
+  case STRSXP:
+  case CHARSXP:
+    g_value_init(raw, G_TYPE_STRING);
+    g_value_set_string(raw, asCString(STRING_ELT(sval, n)));
     break;
-    default:
-      /*fprintf(stderr, "Unhandled R type %d\n", TYPEOF(sval));fflush(stderr);*/
-      return(FALSE);
+  case VECSXP:
+    initGValueFromSValue(VECTOR_ELT(sval, n), raw);
+    break;
+  default:
+    /*fprintf(stderr, "Unhandled R type %d\n", TYPEOF(sval));fflush(stderr);*/
+    return(FALSE);
   }
-	return(TRUE);
+  return(TRUE);
 }
 
 GValue*
@@ -1706,3 +1731,72 @@ asRGSListWithSink(GSList *gslist, const gchar* type) {
     UNPROTECT(1);
     return list;
 }
+
+/* Attempt to override GValue's double->string conversion to use R's logic */
+
+void transformDoubleString(const GValue *src, GValue *dst) {
+  int w, d, e;
+  double n = g_value_get_double(src);
+  formatReal(&n, 1, &w, &d, &e, 0);
+  // could get OutDec, but what about speed?
+  // Outdec = CHAR(asChar(GetOption(install("OutDec"), R_BaseEnv)))[0];
+  const char *formatStr = EncodeReal(n, w, d, e, '.');
+  g_value_set_string(dst, formatStr);
+}
+
+void transformIntString(const GValue *src, GValue *dst) {
+  int w;
+  int n = g_value_get_int(src);
+  formatInteger(&n, 1, &w);
+  g_value_set_string(dst, EncodeInteger(n, w));
+}
+
+void transformBooleanString(const GValue *src, GValue *dst) {
+  int w;
+  gboolean n = g_value_get_boolean(src);
+  formatLogical(&n, 1, &w);
+  g_value_set_string(dst, EncodeLogical(n, w));
+}
+
+/* GLib enum runtime type info support (needed by GIO) */
+
+GType
+g_seek_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { G_SEEK_CUR, "G_SEEK_CUR", "cur" },
+      { G_SEEK_SET, "G_SEEK_SET", "set" },
+      { G_SEEK_END, "G_SEEK_END", "end" },
+      { 0, NULL, NULL }
+    };
+    etype = 
+      g_flags_register_static (g_intern_static_string ("GSeekType"), values);
+  }
+
+  return etype;
+}
+
+
+GType
+g_io_condition_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+      static const GFlagsValue values[] = {
+        { G_IO_IN, "G_IO_IN", "in" },
+        { G_IO_OUT, "G_IO_OUT", "out" },
+        { G_IO_PRI, "G_IO_PRI", "pri" },
+        { G_IO_ERR, "G_IO_ERR", "err" },
+        { G_IO_HUP, "G_IO_HUP", "hup" },
+        { G_IO_NVAL, "G_IO_NVAL", "nval" }, 
+        { 0, NULL, NULL }
+      };
+      etype =
+        g_flags_register_static (g_intern_static_string ("GIOCondition"),
+                                 values);
+    }
+
+  return etype;
+}
